# This file is part of LTSP, https://ltsp.org
# Copyright 2019-2020 the LTSP team, see AUTHORS
# SPDX-License-Identifier: GPL-3.0-or-later

# Handle tasks related to display managers.
# Patching arbitrary .conf files is hard; let's try appending the sections
# we want to the existing files and hope this properly overrides
# @LTSP.CONF: AUTOLOGIN RELOGIN GDM3_CONF LIGHTDM_CONF SDDM_CONF SLICK_CONF
# @LTSP.CONF: LTSPDM_USERS

display_manager_main() {
    local _ALUSER _AUTOLOGIN _RELOGIN

    if [ -z "$AUTOLOGIN" ]; then
        _ALUSER=
    elif ! _ALUSER=$(echo "$HOSTNAME" | sed "s/$AUTOLOGIN/" 2>/dev/null); then
        # AUTOLOGIN can either be a valid sed regex or a username
        _ALUSER=$AUTOLOGIN
    fi
    if [ -n "$_ALUSER" ]; then
        _AUTOLOGIN=true
    else
        _AUTOLOGIN=false
    fi
    if [ "$RELOGIN" = "0" ]; then
        _RELOGIN=false
    else
        _RELOGIN=true
    fi
    re configure_gdm
    re configure_lightdm
    re configure_sddm
    re ltspdm
}

configure_sddm() {
    is_command sddm || return 0
    # Defining a session is required for autologin to work in sddm
    for session in /usr/share/xsessions/plasma.desktop \
        /usr/share/xsessions/*.desktop
    do
        test -f "$session" && break
    done
    if [ -f "$session" ]; then
        session=${session#/usr/share/xsessions/}
    else
        session=
    fi
    {
        echo "
# Generated by \`ltsp init\`, see man:ltsp(8)
# You can append content here by specifying the SDDM_CONF parameter
[Autologin]
User=$_ALUSER
Session=$session
Relogin=$_RELOGIN"
        if [ -f "$SDDM_CONF" ]; then
            re cat "$SDDM_CONF"
        elif [ -n "$SDDM_CONF" ]; then
            echo "$SDDM_CONF"
        fi
    } >/etc/sddm.conf
}

